package cz.zcu.courseware.navigace;

import org.springframework.jdbc.core.namedparam.NamedParameterJdbcDaoSupport;
import org.springframework.dao.DataAccessException;
import org.apache.log4j.Logger;

import java.util.List;
import java.util.Map;
import java.util.HashMap;

public class NavigaceSTAGDAOImpl extends NamedParameterJdbcDaoSupport implements NavigaceSTAGDAO {
  private static final Logger logger = Logger.getLogger(NavigaceSTAGDAOImpl.class);
  private static final boolean isDebugEnabled = logger.isDebugEnabled();

  private static final String SELECT_FAKULTY = "Select c.RV_ABBREVIATION as Zkratka,  c.RV_LOW_VALUE as Key,  c.PORADI, " +
          " DECODE(:lang, 'cs', c.RV_MEANING, c.RV_MEANING) AS nazev From  cg_ref_codes c Where  c.RV_DOMAIN = 'FAKULTA' order by  nazev";
  private static final String GET_FAKULTA_INFO = "Select  c.RV_ABBREVIATION as Zkratka,  c.RV_LOW_VALUE as Key,  c.PORADI,  " +
          "DECODE(:lang, 'cs', c.RV_MEANING, c.RV_MEANING) AS nazev From  cg_ref_codes c Where  c.RV_DOMAIN = 'FAKULTA' AND c.RV_ABBREVIATION = :fakulta";
  private static final String SELECT_KATEDRY = " select ZKR, DECODE(:lang, 'cs', NAZEV, 'sk', NAZEV, NVL(AN_NAZEV, NAZEV)) AS   NAZEV,  " +
          " decode(NAD_PRAC_ZKR, ZKR, null, NAD_PRAC_ZKR) as nad_prac_zkr from V_PRACOVISTE_PLATNE  where typ_prac = 'K' AND nad_prac_zkr = :fakulta order by nazev ";
  private static final String GET_KATEDRA_INFO = "select  DECODE(:lang, 'cs', cp.NAZEV, cp.NAZEV) AS nazev, cp.NAD_PRAC_ZKR as Fk_Nadraz_prac, " +
          "cp2.NAZEV as Nadraz_prac from V_PRACOVISTE_PLATNE cp, V_PRACOVISTE_PLATNE cp2 where cp.ZKR = upper(:zkr_kat) and cp2.ZKR = cp.NAD_PRAC_ZKR ";
  private static final String SELECT_PREDMETY_KATEDRY = "select distinct  p.prac_zkr as pk_prac_zkr,   p.zkr_predm as pk_zkr_predm,   " +
          " DECODE(:lang, 'cs', p.cz_nazev, 'sk', p.cz_nazev, NVL(p.an_nazev, p.cz_nazev)) AS nazev_predmetu " +
          " from pred_varianty p where upper(p.PRAC_ZKR) like upper(:katedra) and upper(p.zkr_predm) like upper(:predmet)  And p.ROK_VARIANTY = ( " +
          " select min(ROK_VARIANTY) from pred_varianty pv2 where pv2.prac_zkr=p.prac_zkr and pv2.zkr_predm=p.zkr_predm and pv2.ROK_VARIANTY in (:rok, to_char(:rok+1)) " +
          " group by prac_zkr,zkr_predm) order by p.ZKR_PREDM";
  private static final String GET_PREDMET_INFO = "select * from (select distinct rownum r, p.prac_zkr as pk_prac_zkr, p.zkr_predm as pk_zkr_predm, " +
          " DECODE(:lang, 'cs', p.cz_nazev, 'sk', p.cz_nazev, NVL(p.an_nazev, p.cz_nazev)) AS nazev_predmetu from pred_varianty p where p.PRAC_ZKR like :katedra " +
          " and p.ZKR_PREDM like :predmet And p.ROK_VARIANTY in (:rok, to_char(:rok+1))) where r = 1";
  private static final String SELECT_PROGRAMY_FROM_FAKULTA = "select sp.STPRIDNO as pk_stpridno, DECODE(:lang, 'cs', sp.cz_nazev, 'sk', sp.cz_nazev, " +
          " NVL(sp.an_nazev, sp.cz_nazev)) AS nazev, DECODE(:lang, 'cs', cob.RV_MEANING, 'sk', cob.RV_MEANING, NVL(cob.ENGLISH, cob.RV_MEANING)) AS typ, " +
          " DECODE(:lang, 'cs', cfo.RV_MEANING, 'sk', cfo.RV_MEANING, NVL(cfo.ENGLISH, cfo.RV_MEANING)) AS forma from Studijni_programy sp, cg_ref_codes cob, " +
          " cg_ref_codes cfo  where sp.FAKULTA_SP = upper(:fakulta)    and  nvl(sp.platny_od, '1970' ) <= nvl(:rok,TO_CHAR(ADD_MONTHS(SYSDATE,-10),'RRRR'))" +
          " and nvl(sp.neplatny_od, '2099') > nvl(:rok,TO_CHAR(ADD_MONTHS(SYSDATE,-10),'RRRR')) and cob.rv_domain = 'TYP_OBORU' and sp.typ = cob.rv_low_value " +
          " and cfo.rv_domain = 'FORMA_OBORU_NEW' and sp.forma = cfo.rv_low_value order by DECODE(:lang, 'cs', sp.cz_nazev, 'sk', sp.cz_nazev, NVL(sp.an_nazev, sp.cz_nazev)), " +
          " DECODE(:lang, 'cs', cob.RV_MEANING, 'sk', cob.RV_MEANING, NVL(cob.ENGLISH, cob.RV_MEANING)), DECODE(:lang, 'cs', cfo.RV_MEANING, 'sk', cfo.RV_MEANING, NVL(cfo.ENGLISH, cfo.RV_MEANING)) ";
  private static final String GET_STUDIJNI_PROGARM_INFO = "select DECODE(:lang, 'cs', sp.cz_nazev, 'sk', sp.cz_nazev, NVL(sp.an_nazev, sp.cz_nazev)) AS nazev, " +
          " DECODE(:lang, 'cs', cf.RV_MEANING, 'sk', cf.RV_MEANING, NVL(cf.ENGLISH, cf.RV_MEANING)) AS fakulta,   " +
          " DECODE(:lang, 'cs', cob.RV_MEANING, 'sk', cob.RV_MEANING, NVL(cob.ENGLISH, cob.RV_MEANING)) AS typ,   " +
          " DECODE(:lang, 'cs', cfo.RV_MEANING, 'sk', cfo.RV_MEANING, NVL(cfo.ENGLISH, cfo.RV_MEANING)) AS forma, " +
          " DECODE(:lang, 'cs', ct.RV_MEANING, 'sk', ct.RV_MEANING, NVL(ct.ENGLISH, ct.RV_MEANING)) AS titul, sp.FAKULTA_SP as fk_fakulta, " +
          " sp.TYP as fk_typ, sp.FORMA as fk_forma, sp.STAND_DELKA, sp.MAX_DELKA, sp.TITUL as fk_titul, ct.RV_ABBREVIATION as titul_zrk, " +
          " sp.LIMIT_CRD as limit_kreditu, sp.VYKAZOVAN, sp.CELOZIVOTNI_VZDELAVANI, sp.PLATNY_OD, sp.NEPLATNY_OD, " +
          " DECODE(:lang, 'cs', sp.CZ_CILE, 'sk', sp.CZ_CILE, NVL(sp.AN_CILE, sp.CZ_CILE)) AS cile, kodsp_i as pk_kodsp_i " +
          " from studijni_programy sp, cg_ref_codes cob, cg_ref_codes cfo, cg_ref_codes ct, cg_ref_codes cf where sp.STPRIDNO = :idno and cob.rv_domain = 'TYP_OBORU' " +
          " and sp.typ = cob.rv_low_value and cfo.rv_domain = 'FORMA_OBORU_NEW' and sp.forma = cfo.rv_low_value and ct.rv_domain = 'TITUL' " +
          " and sp.TITUL = ct.RV_LOW_VALUE and cf.rv_domain = 'FAKULTA' and sp.FAKULTA_SP = cf.RV_LOW_VALUE ";
  private static final String SELECT_OBORY_BY_PROGRAM = "SELECT OB.oboridno, OB.CISOBORU||'/'||OB.CISSPEC AS kod_oboru, OB.ZKRATKA, " +
          " DECODE(:lang, 'cs', OB.CZ_NAZEV, 'sk', OB.CZ_NAZEV, NVL(OB.AN_NAZEV, OB.CZ_NAZEV)) AS nazev FROM obory OB, studijni_programy SP " +
          " WHERE  SP.stpridno = OB.stpridno AND OB.STPRIDNO = :idno AND OB.NEPLATNY_OD is null order by DECODE(:lang, 'cs', OB.CZ_NAZEV, 'sk', OB.CZ_NAZEV, NVL(OB.AN_NAZEV, OB.CZ_NAZEV))";
  private static final String GET_OBOR_INFO = "select DECODE(:lang, 'cs', o.CZ_NAZEV, 'sk', o.CZ_NAZEV, NVL(o.AN_NAZEV, o.CZ_NAZEV)) AS nazev, " +
          " DECODE(:lang, 'cs', cf.RV_MEANING, 'sk', cf.RV_MEANING, NVL(cf.ENGLISH, cf.RV_MEANING)) AS fakulta, " +
          " DECODE(:lang, 'cs', o.CZ_ANOTACE, 'sk', o.CZ_ANOTACE, NVL(o.AN_ANOTACE, o.CZ_ANOTACE)) AS anotace, " +
          " DECODE(:lang, 'cs', cto.RV_MEANING, 'sk', cto.RV_MEANING, NVL(cto.ENGLISH, cto.RV_MEANING)) AS typ, " +
          " DECODE(:lang, 'cs', cfo.RV_MEANING, 'sk', cfo.RV_MEANING, NVL(cfo.ENGLISH, cfo.RV_MEANING)) AS forma, sp.forma as fk_forma, sp.typ as fk_typ, " +
          " o.cisoboru, o.cisspec, o.platny_od, o.neplatny_od, o.oboridno as pk_oboridno, o.FAKULTA_OBORU as fk_fakulta, o.LIMIT_CRD_OBOR as limit_kerditu, " +
          " o.POCET_ETAP, o.MAX_DELKA, o.STAND_DELKA from obory o, cg_ref_codes cfo, cg_ref_codes cto, studijni_programy sp, cg_ref_codes cf where o.OBORIDNO = :idno " +
          " and cfo.rv_domain = 'FORMA_OBORU_NEW' and  sp.forma = cfo.rv_low_value and cto.rv_domain = 'TYP_OBORU' and sp.typ = cto.rv_low_value and " +
          " o.stpridno = sp.stpridno and cf.rv_domain = 'FAKULTA' and o.FAKULTA_OBORU = cf.RV_LOW_VALUE ";
  private static final String GET_AKTUALNI_ROK = "select ROK_PLATNOSTI from KALENDAR where datum = trunc(SYSDATE)";
  private static final String SELECT_PREDMETY_OBORU = "SELECT distinct PV.PRAC_ZKR, PV.ZKR_PREDM, PV.ROK_VARIANTY, " +
          " DECODE(:lang, 'cs', PV.CZ_NAZEV, 'sk', PV.cz_nazev, NVL(PV.AN_NAZEV, PV.CZ_NAZEV)) as nazev FROM install2.studijni_programy SP, install2.obory OB, " +
          " install2.studijni_plany PL, install2.poradi_segmentu PS, install2.segmenty_st_planu SSP, install2.bloky BL, install2.bloky_predm BP, " +
          " install2.pred_varianty PV WHERE SP.stpridno = OB.stpridno AND OB.oboridno = PL.oboridno AND PL.STPLIDNO = PS.STPLIDNO AND PS.SESPIDNO = SSP.SESPIDNO " +
          " AND SSP.SESPIDNO = BL.SESPIDNO AND BL.BLOKIDNO = BP.BLOKIDNO AND BP.PRAC_ZKR = PV.PRAC_ZKR AND BP.ZKR_PREDM = PV.ZKR_PREDM AND BP.ROK_VARIANTY = PV.ROK_VARIANTY " +
          " AND OB.OBORIDNO = :idno AND PL.ROK_PLATNOSTI = SSP.ROK_PLATNOSTI  AND PL.ROK_PLATNOSTI = BL.ROK_PLATNOSTI   AND PL.ROK_PLATNOSTI = :rok";
  private static final String SELECT_PREDMETY_UCITELE = " select a.pk_prac_zkr, a.pk_zkr_predm, decode(:lang, 'cs', b.cz_nazev, 'sk', b.cz_nazev, b.an_nazev) as NAZEV_PREDMETU from   " +
          "    ((           " +
          "    SELECT UPX.Prac_Zkr AS PK_PRAC_ZKR, UPX.Zkr_Predm AS PK_ZKR_PREDM  " +
          "    FROM UCITELE_PREDMETU UPX " +
          "    WHERE UPX.UcitIDNO = :ucitidno AND UPX.ROK_VARIANTY in (:rok, :rok+1)  " +
          "    ) union ( " +
          "    SELECT ra.Prac_Zkr AS PK_PRAC_ZKR, ra.Zkr_Predm AS PK_ZKR_PREDM  " +
          "    FROM Rozvrh_Akce RA " +
          "    WHERE RA.UcitIDNO = :ucitidno and RA.ROK_VARIANTY in (:rok, :rok+1)  " +
          "    ) union ( " +
          "    SELECT RA.Prac_Zkr AS PK_PRAC_ZKR, RA.Zkr_Predm AS PK_ZKR_PREDM  " +
          "    FROM Rozvrh_Akce RA, UC_Rozvrhove_Akce URA  " +
          "    WHERE URA.RoAkIDNO = RA.RoAkIDNO  " +
          "    AND URA.UcitIDNO = :ucitidno  " +
          "    and RA.ROK_VARIANTY in (:rok, :rok+1)  " +
          "    )) a, pred_varianty b " +
          "where a.pk_prac_zkr = b.prac_zkr and a.pk_zkr_predm = b.zkr_predm " +
          "and b.ROK_VARIANTY  = (   " +
          "    select min(ROK_VARIANTY)  " +
          "    from pred_varianty pv2  " +
          "    where pv2.prac_zkr = b.prac_zkr  " +
          "    and pv2.zkr_predm = b.zkr_predm  " +
          "    and pv2.ROK_VARIANTY in (:rok, :rok+1)  " +
          "    group by prac_zkr, zkr_predm " +
          "    )";

  public List selectFakulty(String lang) {
    Map pars = new HashMap(1);
    pars.put("lang", lang);
    return getNamedParameterJdbcTemplate().queryForList(SELECT_FAKULTY, pars);
  }

  public Map getFakultaInfo(String fakulta, String lang) {
    try {
      Map pars = new HashMap(2);
      pars.put("lang", lang);
      pars.put("fakulta", fakulta);
      return getNamedParameterJdbcTemplate().queryForMap(GET_FAKULTA_INFO, pars);
    } catch (DataAccessException e) {
      logger.warn("Nenalezeno fakulta info: "+fakulta+":"+lang);
      return null;
    }
  }

  public List selectKatedry(String fakulta, String lang) {
    Map pars = new HashMap(2);
    pars.put("lang", lang);
    pars.put("fakulta", fakulta);
    return getNamedParameterJdbcTemplate().queryForList(SELECT_KATEDRY, pars);
  }

  public Map getKatedraInfo(String katedra, String lang) {
    try {
      Map pars = new HashMap(2);
      pars.put("lang", lang);
      pars.put("zkr_kat", katedra);
      return getNamedParameterJdbcTemplate().queryForMap(GET_KATEDRA_INFO, pars);
    } catch (DataAccessException e) {
      logger.warn("Nenalezena katedra info: "+katedra+":"+lang);
      return null;
    }
  }

  public List selectPredmetyKatedry(String katedra, String predmet, String rok, String lang) {
    Map pars = new HashMap(4);
    pars.put("lang", lang);
    pars.put("katedra", katedra);
    pars.put("predmet", predmet);
    pars.put("rok", rok);
    return getNamedParameterJdbcTemplate().queryForList(SELECT_PREDMETY_KATEDRY, pars);
  }

  public Map getPredmetInfo(String katedra, String predmet, String rok, String lang) {
    try {
      Map pars = new HashMap(4);
      pars.put("lang", lang);
      pars.put("katedra", katedra);
      pars.put("predmet", predmet);
      pars.put("rok", rok);
      return getNamedParameterJdbcTemplate().queryForMap(GET_PREDMET_INFO, pars);
    } catch (DataAccessException e) {
      logger.warn("Nenalezeno predmet info: "+katedra+"/"+predmet+":"+rok+":"+lang);
      return null;
    }
  }

  public List selectStudijniProgramyFromFakulta(String fakulta, String rok, String lang) {
    Map pars = new HashMap(3);
    pars.put("lang", lang);
    pars.put("fakulta", fakulta);
    pars.put("rok", rok);
    return getNamedParameterJdbcTemplate().queryForList(SELECT_PROGRAMY_FROM_FAKULTA, pars);
  }

  public Map getStudijniProgramInfo(long idno, String lang) {
    Map pars = new HashMap(2);
    pars.put("lang", lang);
    pars.put("idno", new Long(idno));
    try {
      return getNamedParameterJdbcTemplate().queryForMap(GET_STUDIJNI_PROGARM_INFO, pars);
    } catch (DataAccessException e) {
      logger.warn("nenalezeno program info: "+idno+":"+lang);
      return null;
    }
  }

  public List selectStudijniOborByProgramIdno(long idno, String lang) {
    Map pars = new HashMap(2);
    pars.put("lang", lang);
    pars.put("idno", new Long(idno));
    return getNamedParameterJdbcTemplate().queryForList(SELECT_OBORY_BY_PROGRAM, pars);
  }

  public Map getStudijniOborInfo(long idno, String lang) {
    try {
      Map pars = new HashMap(2);
      pars.put("lang", lang);
      pars.put("idno", new Long(idno));
      return getNamedParameterJdbcTemplate().queryForMap(GET_OBOR_INFO, pars);
    } catch (DataAccessException e) {
      logger.warn("nenalezeno obor info:"+idno+":"+lang);
      return null;
    }
  }

  public String getAktualniRok() {
    return (String) getJdbcTemplate().queryForObject(GET_AKTUALNI_ROK, String.class);
  }

  public List selectPredmetyOboru(long idno, String rok, String lang) {
    Map pars = new HashMap(3);
    pars.put("lang", lang);
    pars.put("idno", new Long(idno));
    pars.put("rok", rok);
    return getNamedParameterJdbcTemplate().queryForList(SELECT_PREDMETY_OBORU, pars);
  }

  public List selectPredmetyUcitele(Long ucitidno, String aktAkRok, String language) {
    Map pars = new HashMap(3);
    pars.put("ucitidno", ucitidno);
    pars.put("rok", aktAkRok);
    pars.put("lang", language);
    return  getNamedParameterJdbcTemplate().queryForList(SELECT_PREDMETY_UCITELE, pars);
  }
}
